﻿property pTitle : "Switch to Concentration Context"
property pVer : "0.03"
property pRootMacContext : "Mac"

-- 	Copyright © 2010, Robin Trew
--  All rights reserved.
-- 	
-- 	Redistribution and use in source and binary forms, with or without modification, 
-- 	are permitted provided that the following conditions are met:
-- 	
-- 		- Redistributions of source code must retain the above copyright notice, 
-- 		  this list of conditions and the following disclaimer.
-- 		- Redistributions in binary form must reproduce the above copyright notice, 
-- 		  this list of conditions and the following disclaimer in the documentation 
-- 		  and/or other materials provided with the distribution.
-- 		
-- 	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
-- 	"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
-- 	THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
-- 	IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR 
-- 	ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
-- 	 (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
-- 	 LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
-- 	 WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
-- 	 ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

-- OVERVIEW
-- Launches the Concentrate (getconcentrating.com) activity whose name matches that of the context of the selected OmniFocus item.
-- (Closing and/or opening the specified set of applications, running the specified preparatory script, changing the desktop background etc)


-- If the first selected item is a task or project, the name of its context is used.
-- If a context is selected, then name of that context is used
-- If selected task/project has a non zero value for estimated minutes, the duration of the Concentrate activity is set to this value

-- If the selected item has no context, or the name of its context does not match any activity in Concentrate
-- 1. 	The script ensures that OF has a matching Context for any Activity currently defined in Concentrate
--		(Any new contexts that are required are created as children of the Context named in pRootMacContext above).
-- 2.	The script lists offers a choice of available Concentrate Activities. 
--		If a choice is made:
--		- The chosen Concentrate activity is launched
--		- If the selected task/project has no assigned context, the corresponding context is assigned to it.

-- REQUIREMENTS & INSTALLATION
--  1. Install a trial or licensed copy of 'Concentrate' from getconcentrating.com
-- 2. Use 'Concentrate' to define some working environments on your mac 
--		(applications to or documents automatically open or close, sites to automatically block or open, 
--		 documents to open, preparatory scripts to run etc. etc. )
-- 3. Edit the property pRootMacContext at the start of this script, to name the parent context for your different computing contexts
-- 4. Optionally flag some tasks/projects with contexts whose names match (exactly) the names of 'Concentrate' activities
-- 5. Place this script on the OF toolbar, or attach it to a keystroke, and run it.


on run
	set {strContext, lngMins} to SelectedContextAndMins()
	if strContext ≠ "" then RestoreComputerContext(strContext, lngMins)
end run

on SelectedContextAndMins()
	tell application id "OFOC"
		tell default document
			
			-- IS ANYTHING SELECTED IN THE OMNIFOCUS GUI ?
			if (count of document windows) < 1 then return {"", missing value}
			tell front document window
				set lstSeln to value of selected trees of content where (class of its value is task) or (class of its value is inbox task) or (class of its value is project)
				if length of lstSeln is 0 then set lstSeln to value of selected trees of sidebar where (class of its value is project) or (class of its value is context)
			end tell
			if length of lstSeln < 1 then return {"", missing value}
			set oSeln to first item of lstSeln
			
			-- DO WE HAVE A CONTEXT NAME ?  (AND PERHAPS A DURATION ?)
			tell oSeln
				set cClass to its class
				if (cClass is not context) then
					set oContext to its context
					if oContext is missing value then
						set strContext to ""
					else
						set strContext to name of oContext
					end if
					set lngMins to its estimated minutes
				else -- Class of selected item is context
					set strContext to its name
					set lngMins to missing value
				end if
			end tell
			
			-- IF WE DON'T YET HAVE A CONTEXT NAME ...
			if strContext = "" then
				-- ENSURE THAT OMNIFOCUS HAS A MATCHING CONTEXT FOR EACH ACTIVITY CURRENTLY DEFINED IN 'CONCENTRATE'
				tell application id "com.roobasoft.concentrate"
					set lstActivNames to name of activities
				end tell
				if length of lstActivNames < 1 then
					display dialog "No Activities currently defined in Concentrate" buttons {"OK"} with title pTitle & tab & "Ver. " & pVer
					return {"", missing value}
				end if
				
				try
					set oRootMacContext to first flattened context where name = pRootMacContext
				on error
					set oRootMacContext to make context with properties {name:pRootMacContext}
				end try
				
				set lstOFContexts to name of flattened contexts
				set refChildContexts to a reference to contexts of oRootMacContext
				repeat with oActiv in lstActivNames
					if not (lstOFContexts contains oActiv) then make new context at end of refChildContexts with properties {name:oActiv}
				end repeat
				
				-- AND OFFER THE USER A CHOICE OF THE CURRENTLY DEFINED 'CONCENTRATE' ACTIVITIES
				set varChoice to choose from list lstActivNames with prompt "Launch a Concentrate activity" default items {item 1 of lstActivNames} with title pTitle & tab & "Ver. " & pVer
				if varChoice is false then return {"", missing value}
				set strContext to first item of varChoice
				
				if cClass ≠ context then
					if context of oSeln is missing value then set context of oSeln to first flattened context where name = strContext
				end if
			end if
		end tell
	end tell
	return {strContext, lngMins}
end SelectedContextAndMins

on RestoreComputerContext(strContext, lngMins)
	-- See whether that context matches a concentrate activity
	tell application id "com.roobasoft.concentrate"
		activate
		set lstActivs to activities where name = strContext
		if length of lstActivs > 0 then
			concentrate first item of lstActivs
			if lngMins is not missing value then set minutes remaining to lngMins
		else
			display dialog "None of the activities currently defined in Concentrate match the context name:" & ¬
				return & return & tab & strContext buttons {"OK"} with title pTitle & tab & "Ver. " & pVer
		end if
	end tell
end RestoreComputerContext




